// This file declares the IEnumSTATPROPSTG Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIEnumSTATPROPSTG : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIEnumSTATPROPSTG);
	static IEnumSTATPROPSTG *GetI(PyObject *self);
	static PyComEnumTypeObject type;

	// The Python methods
	static PyObject *Next(PyObject *self, PyObject *args);
	static PyObject *Skip(PyObject *self, PyObject *args);
	static PyObject *Reset(PyObject *self, PyObject *args);
	static PyObject *Clone(PyObject *self, PyObject *args);

protected:
	PyIEnumSTATPROPSTG(IUnknown *pdisp);
	~PyIEnumSTATPROPSTG();
};

// ---------------------------------------------------
//
// Gateway Declaration
class PyGEnumSTATPROPSTG : public PyGatewayBase, public IEnumSTATPROPSTG
{
protected:
	PyGEnumSTATPROPSTG(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT2(PyGEnumSTATPROPSTG, IEnumSTATPROPSTG, IID_IEnumSTATPROPSTG, PyGatewayBase)

	// IEnumSTATPROPSTG
	STDMETHOD(Next)(
		ULONG celt,
		STATPROPSTG * rgelt,
		ULONG * pceltFetched);

	STDMETHOD(Skip)(
		ULONG celt);

	STDMETHOD(Reset)(
		void);

	STDMETHOD(Clone)(
		IEnumSTATPROPSTG ** ppenum);

};