// This file implements the ITransferMediumItem Interface and Gateway for Python.
// Generated by makegw.py

#include "shell_pch.h"
#include "PyIRelatedItem.h"
#include "PyITransferMediumItem.h"

// @doc - This file contains autoduck documentation
// ---------------------------------------------------
//
// Interface Implementation

PyITransferMediumItem::PyITransferMediumItem(IUnknown *pdisp):
	PyIRelatedItem(pdisp)
{
	ob_type = &type;
}

PyITransferMediumItem::~PyITransferMediumItem()
{
}

/* static */ ITransferMediumItem *PyITransferMediumItem::GetI(PyObject *self)
{
	return (ITransferMediumItem *)PyIRelatedItem::GetI(self);
}

// @object PyITransferMediumItem|Description of the interface
static struct PyMethodDef PyITransferMediumItem_methods[] =
{
	{ NULL }
};

PyComTypeObject PyITransferMediumItem::type("PyITransferMediumItem",
		&PyIRelatedItem::type,		// @base PyITransferMediumItem|PyIRelatedItem
		sizeof(PyITransferMediumItem),
		PyITransferMediumItem_methods,
		GET_PYCOM_CTOR(PyITransferMediumItem));
// ---------------------------------------------------
//
// Gateway Implementation
