// This file declares the ITransferMediumItem Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyITransferMediumItem : public PyIRelatedItem
{
public:
	MAKE_PYCOM_CTOR(PyITransferMediumItem);
	static ITransferMediumItem *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods

protected:
	PyITransferMediumItem(IUnknown *pdisp);
	~PyITransferMediumItem();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGTransferMediumItem : public PyGRelatedItem, public ITransferMediumItem
{
protected:
	PyGTransferMediumItem(PyObject *instance) : PyGRelatedItem(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT2(PyGTransferMediumItem, ITransferMediumItem, IID_ITransferMediumItem, PyGRelatedItem)

	// Only has IRelatedItem methods
	STDMETHOD(GetItemIDList)(PIDLIST_ABSOLUTE * ppidl)
		{return PyGRelatedItem::GetItemIDList(ppidl);}

	STDMETHOD(GetItem)(IShellItem ** ppsi)
		{return PyGRelatedItem::GetItem(ppsi);}
};
