// This file declares the IActiveScriptParseProcedure Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIActiveScriptParseProcedure : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIActiveScriptParseProcedure);
	static IActiveScriptParseProcedure *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *ParseProcedureText(PyObject *self, PyObject *args);

protected:
	PyIActiveScriptParseProcedure(IUnknown *pdisp);
	~PyIActiveScriptParseProcedure();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGActiveScriptParseProcedure : public PyGatewayBase, public IActiveScriptParseProcedure
{
protected:
	PyGActiveScriptParseProcedure(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(PyGActiveScriptParseProcedure, IActiveScriptParseProcedure, IID_IActiveScriptParseProcedure)

	// IActiveScriptParseProcedure
	STDMETHOD(ParseProcedureText)(
		LPCOLESTR pstrCode,
		LPCOLESTR pstrFormalParams,
		LPCOLESTR pstrProcedureName,
		LPCOLESTR pstrItemName,
		IUnknown __RPC_FAR * punkContext,
		LPCOLESTR pstrDelimiter,
		DWORD_PTR dwSourceContextCookie,
		ULONG ulStartingLineNumber,
		DWORD dwFlags,
		IDispatch __RPC_FAR *__RPC_FAR * ppdisp);

};
