// This file declares the IObjectWithPropertyKey Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIObjectWithPropertyKey : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIObjectWithPropertyKey);
	static IObjectWithPropertyKey *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *SetPropertyKey(PyObject *self, PyObject *args);
	static PyObject *GetPropertyKey(PyObject *self, PyObject *args);

protected:
	PyIObjectWithPropertyKey(IUnknown *pdisp);
	~PyIObjectWithPropertyKey();
};
