// This file implements the IIdentityName Interface and Gateway for Python.
// Generated by makegw.py

#include "shell_pch.h"
#include "PyIRelatedItem.h"
#include "PyIIdentityName.h"

// @doc - This file contains autoduck documentation
// ---------------------------------------------------
//
// Interface Implementation

PyIIdentityName::PyIIdentityName(IUnknown *pdisp):
	PyIRelatedItem(pdisp)
{
	ob_type = &type;
}

PyIIdentityName::~PyIIdentityName()
{
}

/* static */ IIdentityName *PyIIdentityName::GetI(PyObject *self)
{
	return (IIdentityName *)PyIRelatedItem::GetI(self);
}

// @object PyIIdentityName|Description of the interface
static struct PyMethodDef PyIIdentityName_methods[] =
{
	{ NULL }
};

PyComTypeObject PyIIdentityName::type("PyIIdentityName",
		&PyIRelatedItem::type, // @base PyIIdentityName|PyIRelatedItem
		sizeof(PyIIdentityName),
		PyIIdentityName_methods,
		GET_PYCOM_CTOR(PyIIdentityName));
// ---------------------------------------------------
//
// Gateway Implementation
