// This file implements the ICurrentItem Interface and Gateway for Python.
// Generated by makegw.py

#include "shell_pch.h"
#include "PyIRelatedItem.h"
#include "PyICurrentItem.h"

// @doc - This file contains autoduck documentation
// ---------------------------------------------------
//
// Interface Implementation

PyICurrentItem::PyICurrentItem(IUnknown *pdisp):
	PyIRelatedItem(pdisp)
{
	ob_type = &type;
}

PyICurrentItem::~PyICurrentItem()
{
}

/* static */ ICurrentItem *PyICurrentItem::GetI(PyObject *self)
{
	return (ICurrentItem *)PyIRelatedItem::GetI(self);
}

// @object PyICurrentItem|Description of the interface
static struct PyMethodDef PyICurrentItem_methods[] =
{
	{ NULL }
};

PyComTypeObject PyICurrentItem::type("PyICurrentItem",
		&PyIRelatedItem::type,	// @base PyICurrentItem|PyIRelatedItem
		sizeof(PyICurrentItem),
		PyICurrentItem_methods,
		GET_PYCOM_CTOR(PyICurrentItem));
// ---------------------------------------------------
//
// Gateway Implementation
