// This file declares the IBackgroundCopyCallback Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Gateway Declaration

# include "PythonCOMServer.h"

class PyGBackgroundCopyCallback : public PyGatewayBase, public IBackgroundCopyCallback
{
protected:
	PyGBackgroundCopyCallback(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT2(PyGBackgroundCopyCallback, IBackgroundCopyCallback, IID_IBackgroundCopyCallback, PyGatewayBase)



	// IBackgroundCopyCallback
	STDMETHOD(JobTransferred)(
		IBackgroundCopyJob * pJob);

	STDMETHOD(JobError)(
		IBackgroundCopyJob * pJob,
		IBackgroundCopyError * pError);

	STDMETHOD(JobModification)(
		IBackgroundCopyJob * pJob,
		DWORD dwReserved);

};
