// This file declares the IProvideTaskPage Interface for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

#include "_win32com.h"
#include "mstask.h"

class PyIProvideTaskPage : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIProvideTaskPage);
	static IProvideTaskPage *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *GetPage(PyObject *self, PyObject *args);

protected:
	PyIProvideTaskPage(IUnknown *pdisp);
	~PyIProvideTaskPage();
};
