// This file declares the IDebugSyncOperation Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIDebugSyncOperation : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIDebugSyncOperation);
	static IDebugSyncOperation *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *GetTargetThread(PyObject *self, PyObject *args);
	static PyObject *Execute(PyObject *self, PyObject *args);
	static PyObject *InProgressAbort(PyObject *self, PyObject *args);

protected:
	PyIDebugSyncOperation(IUnknown *pdisp);
	~PyIDebugSyncOperation();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGDebugSyncOperation : public PyGatewayBase, public IDebugSyncOperation
{
protected:
	PyGDebugSyncOperation(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(PyGDebugSyncOperation, IDebugSyncOperation, IID_IDebugSyncOperation)

	// IDebugSyncOperation
	STDMETHOD(GetTargetThread)(
		IDebugApplicationThread __RPC_FAR *__RPC_FAR * ppatTarget);

	STDMETHOD(Execute)(
		IUnknown __RPC_FAR *__RPC_FAR * ppunkResult);

	STDMETHOD(InProgressAbort)(
		void);

};
