// This file declares the IActiveScriptError Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PYAXSCRIPT_EXPORT PyIActiveScriptError : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR_ERRORINFO(PyIActiveScriptError, IID_IActiveScriptError);
	static IActiveScriptError *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *GetExceptionInfo(PyObject *self, PyObject *args);
	static PyObject *GetSourcePosition(PyObject *self, PyObject *args);
	static PyObject *GetSourceLineText(PyObject *self, PyObject *args);

protected:
	PyIActiveScriptError(IUnknown *pdisp);
	~PyIActiveScriptError();
};
