#define INITGUID

#define USES_IID_IMAPISession
#define USES_IID_IMAPIStatus
#define USES_IID_IMAPITable
#define USES_IID_IMAPIProp
#define USES_IID_IMAPIFolder
#define USES_IID_IMessage
#define USES_IID_IMsgStore
#define USES_IID_IMAPIContainer
#define USES_IID_IMAPIProgress
#define USES_IID_IAttachment
#define USES_IID_IProfAdmin
#define USES_IID_IMAPIAdviseSink
#define USES_IID_IAddrBook
#define USES_IID_IMAPIPropData
#define USES_IID_IMailUser
#define USES_IID_IDistList
#define USES_IID_IABContainer
#define USES_IID_IProfSect
#define USES_IID_IMsgServiceAdmin
#define USES_IID_IProviderAdmin


#define USES_PS_PUBLIC_STRINGS
#define USES_PS_MAPI
#define USES_PS_ROUTING_EMAIL_ADDRESSES
#define USES_PS_ROUTING_ADDRTYPE
#define USES_PS_ROUTING_DISPLAY_NAME
#define USES_PS_ROUTING_ENTRYID
#define USES_PS_ROUTING_SEARCH_KEY



#include "windows.h"
#include "mapix.h"
#include "mapiguid.h"
