// This file declares the IExternalConnection Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIExternalConnection : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIExternalConnection);
	static IExternalConnection *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *AddConnection(PyObject *self, PyObject *args);
	static PyObject *ReleaseConnection(PyObject *self, PyObject *args);

protected:
	PyIExternalConnection(IUnknown *pdisp);
	~PyIExternalConnection();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGExternalConnection : public PyGatewayBase, public IExternalConnection
{
protected:
	PyGExternalConnection(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(PyGExternalConnection, IExternalConnection, IID_IExternalConnection)

	// IExternalConnection
	virtual DWORD STDMETHODCALLTYPE AddConnection(
		DWORD extconn,
		DWORD reserved);

	virtual DWORD STDMETHODCALLTYPE ReleaseConnection(
		DWORD extconn,
		DWORD reserved,
		BOOL fLastReleaseCloses);

};
