// This file declares the IContextMenu Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
// ---------------------------------------------------
//
// Gateway Declaration
#include "PyIContextMenu2.h"

class PyGContextMenu3 : public PyGContextMenu2, public IContextMenu3
{
protected:
	PyGContextMenu3(PyObject *instance) : PyGContextMenu2(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT2(PyGContextMenu3, IContextMenu3, IID_IContextMenu3, PyGContextMenu2)

	// IContextMenu
	STDMETHOD(QueryContextMenu)(
		HMENU hmenu,
		UINT indexMenu,
		UINT idCmdFirst,
		UINT idCmdLast,
		UINT uFlags);

	STDMETHOD(InvokeCommand)(
		CMINVOKECOMMANDINFO __RPC_FAR * lpici);

	STDMETHOD(GetCommandString)(
		UINT_PTR idCmd,
		UINT uType,
		UINT     __RPC_FAR * pwReserved,
		LPSTR pszName,
		UINT cchMax);
	// IContextMenu2
        STDMETHOD(HandleMenuMsg)( 
                UINT uMsg,
                WPARAM wParam,
                LPARAM lParam);
	// IContextMenu3
        STDMETHOD(HandleMenuMsg2)( 
                UINT uMsg,
                WPARAM wParam,
                LPARAM lParam,
                LRESULT *plResult);
};
