// This file declares the IEnumDebugApplicationNodes Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIEnumDebugApplicationNodes : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR_ERRORINFO(PyIEnumDebugApplicationNodes, IID_IEnumDebugApplicationNodes);
	static IEnumDebugApplicationNodes *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *Next(PyObject *self, PyObject *args);
	static PyObject *Skip(PyObject *self, PyObject *args);
	static PyObject *Reset(PyObject *self, PyObject *args);
	static PyObject *Clone(PyObject *self, PyObject *args);

protected:
	PyIEnumDebugApplicationNodes(IUnknown *pdisp);
	~PyIEnumDebugApplicationNodes();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGEnumDebugApplicationNodes : public PyGatewayBase, public IEnumDebugApplicationNodes
{
protected:
	PyGEnumDebugApplicationNodes(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(PyGEnumDebugApplicationNodes, IEnumDebugApplicationNodes, IID_IEnumDebugApplicationNodes)

	// IEnumDebugApplicationNodes
	STDMETHOD(Next)(
		ULONG celt,
		IDebugApplicationNode __RPC_FAR *__RPC_FAR * pprddp,
		ULONG __RPC_FAR * pceltFetched);

	STDMETHOD(Skip)(
		ULONG celt);

	STDMETHOD(Reset)(
		void);

	STDMETHOD(Clone)(
		IEnumDebugApplicationNodes __RPC_FAR *__RPC_FAR * pperddp);

};
