#ifndef GUIDS_H
#define GUIDS_H


/*
 * FILENAME:
 *	guids.h
 *
 * DESCRIPTION:
 *	GUID definitions for the project.
 */


DEFINE_GUID(IID_IScript,  0x350E5A60L, 0x2F61, 0x11CF, 0xAA, 0xA5, 0x00, 0x40, 0x1C, 0x60, 0x85, 0x01);
DEFINE_GUID(IID_IDispatchEx, 0xa0aac450, 0xa77b, 0x11cf, 0x91, 0xd0, 0x0, 0xaa, 0x0, 0xc1, 0x4a, 0x7c);


// {DF630910-1C1D-11d0-AE36-8C0F5E000000}
DEFINE_GUID(CLSID_PyScript2,  0xdf630910, 0x1c1d, 0x11d0, 0xae, 0x36, 0x8c, 0xf, 0x5e, 0x0, 0x0, 0x0);
// {DF630910-1C1D-11d0-AE36-8C0F5E000000}
static const GUID szCLSID_PyScript2 = 
{ 0xdf630910, 0x1c1d, 0x11d0, { 0xae, 0x36, 0x8c, 0xf, 0x5e, 0x0, 0x0, 0x0 } };

#endif
