// This file declares the IEnumShellItems Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIEnumShellItems : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIEnumShellItems);
	static IEnumShellItems *GetI(PyObject *self);
	static PyComEnumTypeObject type;

	// The Python methods
	static PyObject *Next(PyObject *self, PyObject *args);
	static PyObject *Skip(PyObject *self, PyObject *args);
	static PyObject *Reset(PyObject *self, PyObject *args);
	static PyObject *Clone(PyObject *self, PyObject *args);

protected:
	PyIEnumShellItems(IUnknown *pdisp);
	~PyIEnumShellItems();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGEnumShellItems : public PyGatewayBase, public IEnumShellItems
{
protected:
	PyGEnumShellItems(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT2(PyGEnumShellItems, IEnumShellItems, IID_IEnumShellItems, PyGatewayBase)



	// IEnumShellItems
	STDMETHOD(Next)(
		ULONG celt,
		__RPC__out_ecount_part(celt, *pceltFetched) IShellItem ** rgelt,
		__RPC__out ULONG * pceltFetched);

	STDMETHOD(Skip)(
		ULONG celt);

	STDMETHOD(Reset)(
		void);

	STDMETHOD(Clone)(
		__RPC__deref_out_opt IEnumShellItems ** ppenum);

};
