// This file declares the IActiveScriptErrorDebug Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration
// We need AXScript headers.
#include "AXScript.h"
#include "PyGActiveScriptError.h"
#include "PyIActiveScriptError.h"

class PyIActiveScriptErrorDebug : public PyIActiveScriptError
{
public:
	MAKE_PYCOM_CTOR(PyIActiveScriptErrorDebug);
	static IActiveScriptErrorDebug *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *GetDocumentContext(PyObject *self, PyObject *args);
	static PyObject *GetStackFrame(PyObject *self, PyObject *args);

protected:
	PyIActiveScriptErrorDebug(IUnknown *pdisp);
	~PyIActiveScriptErrorDebug();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGActiveScriptErrorDebug : public PyGActiveScriptError, public IActiveScriptErrorDebug
{
protected:
	PyGActiveScriptErrorDebug(PyObject *instance) : PyGActiveScriptError(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(PyGActiveScriptErrorDebug, IActiveScriptErrorDebug, IID_IActiveScriptErrorDebug)

	// IActiveScriptError
	STDMETHOD(GetExceptionInfo)(EXCEPINFO __RPC_FAR *pexcepinfo);

	STDMETHOD(GetSourcePosition)(DWORD __RPC_FAR *pdwSourceContext, ULONG __RPC_FAR *pulLineNumber, LONG __RPC_FAR *plCharacterPosition);
		
	STDMETHOD(GetSourceLineText)(BSTR __RPC_FAR *pbstrSourceLine);

	// IActiveScriptErrorDebug
	STDMETHOD(GetDocumentContext)(
		IDebugDocumentContext __RPC_FAR *__RPC_FAR * ppssc);

	STDMETHOD(GetStackFrame)(
		IDebugStackFrame __RPC_FAR *__RPC_FAR * ppdsf);

};
