// This file declares the IShellLibrary Interface for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

// Requires Windows 7 SDK to build
#if WINVER >= 0x0601

class PyIShellLibrary : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIShellLibrary);
	static IShellLibrary *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *LoadLibraryFromItem(PyObject *self, PyObject *args);
	static PyObject *LoadLibraryFromKnownFolder(PyObject *self, PyObject *args);
	static PyObject *AddFolder(PyObject *self, PyObject *args);
	static PyObject *RemoveFolder(PyObject *self, PyObject *args);
	static PyObject *GetFolders(PyObject *self, PyObject *args);
	static PyObject *ResolveFolder(PyObject *self, PyObject *args);
	static PyObject *GetDefaultSaveFolder(PyObject *self, PyObject *args);
	static PyObject *SetDefaultSaveFolder(PyObject *self, PyObject *args);
	static PyObject *GetOptions(PyObject *self, PyObject *args);
	static PyObject *SetOptions(PyObject *self, PyObject *args);
	static PyObject *GetFolderType(PyObject *self, PyObject *args);
	static PyObject *SetFolderType(PyObject *self, PyObject *args);
	static PyObject *GetIcon(PyObject *self, PyObject *args);
	static PyObject *SetIcon(PyObject *self, PyObject *args);
	static PyObject *Commit(PyObject *self, PyObject *args);
	static PyObject *Save(PyObject *self, PyObject *args);
	static PyObject *SaveInKnownFolder(PyObject *self, PyObject *args);

protected:
	PyIShellLibrary(IUnknown *pdisp);
	~PyIShellLibrary();
};

#endif // WINVER
