// This file declares the IExplorerPaneVisibility Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIExplorerPaneVisibility : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIExplorerPaneVisibility);
	static IExplorerPaneVisibility *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *GetPaneState(PyObject *self, PyObject *args);

protected:
	PyIExplorerPaneVisibility(IUnknown *pdisp);
	~PyIExplorerPaneVisibility();
};
