// This file declares the IPropertySetStorage Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIPropertySetStorage : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIPropertySetStorage);
	static IPropertySetStorage *GetI(PyObject *self);
	static PyComEnumProviderTypeObject type;

	// The Python methods
	static PyObject *Create(PyObject *self, PyObject *args);
	static PyObject *Open(PyObject *self, PyObject *args);
	static PyObject *Delete(PyObject *self, PyObject *args);
	static PyObject *Enum(PyObject *self, PyObject *args);
protected:
	PyIPropertySetStorage(IUnknown *pdisp);
	~PyIPropertySetStorage();
};

// ---------------------------------------------------
//
// Gateway Declaration

class PyGPropertySetStorage : public PyGatewayBase, public IPropertySetStorage
{
protected:
	PyGPropertySetStorage(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT2(PyGPropertySetStorage, IPropertySetStorage, IID_IPropertySetStorage, PyGatewayBase)



	// IPropertySetStorage
	STDMETHOD(Create)(
		REFFMTID rfmtid,
		const CLSID * pclsid,
		DWORD grfFlags,
		DWORD grfMode,
		IPropertyStorage ** ppprstg);

	STDMETHOD(Open)(
		REFFMTID rfmtid,
		DWORD grfMode,
		IPropertyStorage ** ppprstg);

	STDMETHOD(Delete)(
		REFFMTID rfmtid);

	STDMETHOD(Enum)(
		IEnumSTATPROPSETSTG ** ppenum);

};