// This file declares the ICancelMethodCalls Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyICancelMethodCalls : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyICancelMethodCalls);
	static ICancelMethodCalls *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *Cancel(PyObject *self, PyObject *args);
	static PyObject *TestCancel(PyObject *self, PyObject *args);

protected:
	PyICancelMethodCalls(IUnknown *pdisp);
	~PyICancelMethodCalls();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGCancelMethodCalls : public PyGatewayBase, public ICancelMethodCalls
{
protected:
	PyGCancelMethodCalls(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT2(PyGCancelMethodCalls, ICancelMethodCalls, IID_ICancelMethodCalls, PyGatewayBase)



	// ICancelMethodCalls
	STDMETHOD(Cancel)(
		ULONG ulSeconds);

	STDMETHOD(TestCancel)(
		void);

};
