// This file declares the IDebugDocumentHost Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIDebugDocumentHost : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR_ERRORINFO(PyIDebugDocumentHost, IID_IDebugDocumentHost);
	static IDebugDocumentHost *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *GetDeferredText(PyObject *self, PyObject *args);
	static PyObject *GetScriptTextAttributes(PyObject *self, PyObject *args);
	static PyObject *OnCreateDocumentContext(PyObject *self, PyObject *args);
	static PyObject *GetPathName(PyObject *self, PyObject *args);
	static PyObject *GetFileName(PyObject *self, PyObject *args);
	static PyObject *NotifyChanged(PyObject *self, PyObject *args);

protected:
	PyIDebugDocumentHost(IUnknown *pdisp);
	~PyIDebugDocumentHost();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGDebugDocumentHost : public PyGatewayBase, public IDebugDocumentHost
{
protected:
	PyGDebugDocumentHost(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(PyGDebugDocumentHost, IDebugDocumentHost, IID_IDebugDocumentHost)

	// IDebugDocumentHost
	STDMETHOD(GetDeferredText)(
		DWORD dwTextStartCookie,
		WCHAR __RPC_FAR * pcharText,
		SOURCE_TEXT_ATTR __RPC_FAR * pstaTextAttr,
		ULONG __RPC_FAR * pcNumChars,
		ULONG cMaxChars);

	STDMETHOD(GetScriptTextAttributes)(
		LPCOLESTR pstrCode,
		ULONG uNumCodeChars,
		LPCOLESTR pstrDelimiter,
		DWORD dwFlags,
		SOURCE_TEXT_ATTR __RPC_FAR * pattr);

	STDMETHOD(OnCreateDocumentContext)(
		IUnknown __RPC_FAR *__RPC_FAR * ppunkOuter);

	STDMETHOD(GetPathName)(
		BSTR __RPC_FAR * pbstrLongName,
		BOOL __RPC_FAR * pfIsOriginalFile);

	STDMETHOD(GetFileName)(
		BSTR __RPC_FAR * pbstrShortName);

	STDMETHOD(NotifyChanged)(
		void);

};
