// This file declares the IExplorerBrowser Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIExplorerBrowser : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIExplorerBrowser);
	static IExplorerBrowser *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *Initialize(PyObject *self, PyObject *args);
	static PyObject *Destroy(PyObject *self, PyObject *args);
	static PyObject *SetRect(PyObject *self, PyObject *args);
	static PyObject *SetPropertyBag(PyObject *self, PyObject *args);
	static PyObject *SetEmptyText(PyObject *self, PyObject *args);
	static PyObject *SetFolderSettings(PyObject *self, PyObject *args);
	static PyObject *Advise(PyObject *self, PyObject *args);
	static PyObject *Unadvise(PyObject *self, PyObject *args);
	static PyObject *SetOptions(PyObject *self, PyObject *args);
	static PyObject *GetOptions(PyObject *self, PyObject *args);
	static PyObject *BrowseToIDList(PyObject *self, PyObject *args);
	static PyObject *BrowseToObject(PyObject *self, PyObject *args);
	static PyObject *FillFromObject(PyObject *self, PyObject *args);
	static PyObject *RemoveAll(PyObject *self, PyObject *args);
	static PyObject *GetCurrentView(PyObject *self, PyObject *args);

protected:
	PyIExplorerBrowser(IUnknown *pdisp);
	~PyIExplorerBrowser();
};
