// This file declares the IPropertyStoreCache Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration
#include "_win32com.h"
#include "PyIPropertyStore.h"
#include "propsys.h"
class PyIPropertyStoreCache : public PyIPropertyStore
{
public:
	MAKE_PYCOM_CTOR(PyIPropertyStoreCache);
	static IPropertyStoreCache *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *GetState(PyObject *self, PyObject *args);
	static PyObject *GetValueAndState(PyObject *self, PyObject *args);
	static PyObject *SetState(PyObject *self, PyObject *args);
	static PyObject *SetValueAndState(PyObject *self, PyObject *args);

protected:
	PyIPropertyStoreCache(IUnknown *pdisp);
	~PyIPropertyStoreCache();
};
