// This file declares the IDebugApplicationNodeEvents Interface and Gateway for Python.
// Generated by makegw.py
// ---------------------------------------------------
//
// Interface Declaration

class PyIDebugApplicationNodeEvents : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(PyIDebugApplicationNodeEvents);
	static IDebugApplicationNodeEvents *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
	static PyObject *onAddChild(PyObject *self, PyObject *args);
	static PyObject *onRemoveChild(PyObject *self, PyObject *args);
	static PyObject *onDetach(PyObject *self, PyObject *args);
	static PyObject *onAttach(PyObject *self, PyObject *args);

protected:
	PyIDebugApplicationNodeEvents(IUnknown *pdisp);
	~PyIDebugApplicationNodeEvents();
};
// ---------------------------------------------------
//
// Gateway Declaration

class PyGDebugApplicationNodeEvents : public PyGatewayBase, public IDebugApplicationNodeEvents
{
protected:
	PyGDebugApplicationNodeEvents(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(PyGDebugApplicationNodeEvents, IDebugApplicationNodeEvents, IID_IDebugApplicationNodeEvents)

	// IDebugApplicationNodeEvents
	STDMETHOD(onAddChild)(
		IDebugApplicationNode __RPC_FAR * prddpChild);

	STDMETHOD(onRemoveChild)(
		IDebugApplicationNode __RPC_FAR * prddpChild);

	STDMETHOD(onDetach)(
		void);

	STDMETHOD(onAttach)(
		IDebugApplicationNode __RPC_FAR * prddpParent);

};
